#!/usr/bin/env python

""" Handles Input-Output of ASCII grid (raster) files
    AUTHOR:         Arika Ligmann-Zielinska
    UPDATED:        Apr 27, 2012 
"""
#-------------
import numpy
import os.path

class Raster:

    def __init__(self,asciifile,datatype=float):
        """ in: ASCII grid file full path name, (datatype = int, float)
        """
        self.type = datatype
        self.file = asciifile
        if not os.path.exists(asciifile):
            raise IOError("File "+asciifile+" does not exist.")
        self.name = os.path.basename(asciifile)[:-4]
        self.ncols = None; self.nrows = None
        self.cellsize = None
        self.NODATA_value= None
        self.body = None; self.header_str = None
        try:
            self.getHeader()
            self.getBody()
        except IOError:
            print "Inappropriate file format."
        self.domain = None
        self.setDomain()

    def __str__(self):
        """ string describing ASCII grid
        """
        label = "Name\t\t%s\nRows\t\t%d\nCols\t\t%d\nCell size\t%d\n" \
                % (self.name,self.nrows,self.ncols,self.cellsize)

        return label

    def getHeader(self):
        """ header data 
        """
        f = open(self.file, 'r')
        head = f.readlines()[:6]
        f.close()
        self.header_str = "".join(head)
        data = [line.strip().split() for line in head]
        # populate attributes
        self.ncols = int(data.pop(0)[1])
        self.nrows = int(data.pop(0)[1])
        self.NODATA_value = int(data.pop(-1)[1])
        self.cellsize = int(data.pop(-1)[1])


    def getBody(self):
        """ ASCII main content as 1D numpy array
        """
        f = open(self.file, 'r')
        rawdata = f.readlines()[6:]
        f.close()
        array2D = [map(float,rec.strip().split()) for rec in rawdata]
        self.body = numpy.array(array2D).flatten()

    def setDomain(self):
        """ exhaustive and exclusive list of raster body values

            not useful for FLOAT rasters
        """
        self.domain = list(set(self.body))

    def save(self,newbody,fname,not_correct=False,convert_negatives=True):
        """ Saves new content to a file; in: ASCII body (1D numpy array), outfile name (str) , correct for NODATA (bool)
        """
        f = open(fname,'w')
        f.write(self.header_str)
        if not_correct:
            newbody = numpy.nan_to_num(newbody)
            print fname,"All NaN values were converted to 0.0"
        # convert negatives
        if convert_negatives:
            total_conv = float(numpy.sum(numpy.where(newbody < 0, 1, 0)))
            print "Negatives converted to 0.0.",
            print round((total_conv/newbody.size*100),2),"% of raster area had negative values"
            newbody = numpy.where(newbody<0.0,0.0,newbody)
        newbody = numpy.reshape(newbody,(self.nrows,self.ncols))
        newbody = [map(str,rec) for rec in newbody]
        for rec in newbody:
            f.write(" ".join(rec)+"\n")
        f.close()
        print "Output ASCII grid",fname,"saved."

    
    
