#!/usr/bin/env python

""" Decision Rules (Aggregation Functions)
    AUTHOR:         Arika Ligmann-Zielinska
    UPDATED:        Apr 27, 2012

    NOTE: only Ideal Point implemented
"""
#-------------
import numpy as npy
#import winsound

def ideal_point(rasters, weights):
    """ in:  list of rasters (1D numpy arrays), list of weights (floats)
        out: 1D numpy array of composite scores (Ideal Point map)
    """
    # recalculate weights so that they add up to 1.0
    total = float(npy.sum(weights))
    weightsample = [float(weight)/total for weight in weights]
    # create templates for ideals & nadirs surfaces
    ideal_rasters = npy.zeros(rasters[0].size)
    nadir_rasters = npy.zeros(rasters[0].size)
    # calculations per array
    for n,raster in enumerate(rasters):
        weight = weightsample[n]
        # get ideal and nadir
        ideal=npy.amax(raster);nadir=npy.amin(raster)
        # differences from ideal & nadir
        tmp_ideal = raster-ideal
        tmp_nadir = raster-nadir
        # weight the differences
        tmp_wideal = tmp_ideal*weight
        tmp_wnadir = tmp_nadir*weight
        # square the weighted differences
        ideal_rstr = npy.power(tmp_wideal,2)
        nadir_rstr = npy.power(tmp_wnadir,2)
        # add the ideal & nadir surfaces to composite ideals & nadirs
        ideal_rasters += ideal_rstr
        nadir_rasters += nadir_rstr
    # separations from ideal and nadir
    temp_sq_idraster = npy.sqrt(ideal_rasters)
    temp_sq_ndraster = npy.sqrt(nadir_rasters)
    # sum of separations
    temp_sumIdealNadir = temp_sq_idraster+temp_sq_ndraster
    # IDEAL POINT
    ideal_point_raster = temp_sq_ndraster/temp_sumIdealNadir
    return ideal_point_raster


def ideal_point_radial(N,radial_block_weights,rasters):
    """ in: number of base runs (N: int), radial samples (list of radial weight blocks), list of rasters
        out [generator]: radial Ideal Point maps - numpy 1D arrays [mapA,mapB,k*mapsAb]
    """
    n = 0
    try:
        while n < N:
            completed = round(float(n)/N*100.0,2)
            one_weights_block = radial_block_weights[n]
            IPmaps = [ideal_point(rasters, weights) for weights in one_weights_block]
            yield IPmaps # one radial set of 1D array maps [mapA,mapB,k*mapsAb]
            n += 1
            dont_print_me = completed%5
            if not dont_print_me:
                if completed >= 10:
                    print "completed",completed,"%"
#                    winsound.PlaySound("*", winsound.SND_ALIAS)
                
                
    except GeneratorExit:
        print("Only made it to %d out of %d" % (n,N))
        

#-------------
